/*******************************************************************************
 *                                      *                                      *
 *    HOLDUP BV                         *    STOGGER BV                        *
 *    Nachtegaalstraat 8                *    Haagveld 1A                       *
 *    5988 NG  Helden                   *    5981 PK  Panningen                *
 *    The Netherlands                   *    The Netherlands                   *
 *                                      *                                      *
 *******************************************************************************
 *                                      *                                      *
 *    Aubert Dupont                     *                                      *
 *    aubert@holdup.nl                  *                                      *
 *                                      *                                      *
 *******************************************************************************
 *                                                                             *
 *    Project    -   STOGGER - E-41-2000 - Ventilator PAL                      *
 *    Author     -   Aubert Dupont                                             *
 *    Build Tool -   MPLAB X IDE v4.15 & MPLAB XC8 v1.45                       *
 *    Revision   -   1.01                                                      *
 *    Date       -   2020-04-01                                                *
 *    Function   -   User definition file                                      *
 *    File       -   user.h                                                    *
 *                                                                             *
 *******************************************************************************
 *                                                                             *
 *    (C)  2020  HOLDUP BV         ALL RIGHTS RESERVED                         *
 *    (C)  2020  STOGGER BV        ALL RIGHTS RESERVED                         *
 *                                                                             *
 ******************************************************************************/

/*******************************************************************************
 *
 *   Documentation / Functional description
 *
 *******************************************************************************
 *
 *   For Communication from MASTER to BOOT LOADER, the HEX record data format
 *   is used. For the answer from BOOT LOADER to MASTER, a similar data format
 *   is used for the answer, with same HEX record start and end markers, but not
 *   using any checksum, etc.
 *
 *   The communication specifications:
 *
 *   Clock:   Asynchronous
 *   Mode:    HALF-duplex
 *   Speed:   9600 baud
 *   Size:    8-bits transmission
 *   Parity:  None
 *   Stop:    1-bit
 *
 *
 *   HEX record data layout:
 *   =======================
 *
 *   The HEX record structure used, is a modified version of the Microchip
 *   or Intel standard. The modification consists of an record end marker and
 *   additional type records, to allow for flash erase commands.
 *
 *   The Microchip XC8 compiler generates a record size which is fixed to
 *   a maximum of 42 characters (8 data words).
 *
 *   :BBAAAATTHHHH...HHHHCC;   one single HEX record
 *   -----------------------
 *   :----------------------   the HEX record start marker
 *   -BB--------------------   the NUMBER of data bytes in this record
 *   ---AAAA----------------   the four digit hexadecimal ADDRESS
 *   -------TT--------------   is a two digit record TYPE
 *   ---------HHHH...HHHH---   is a four digit hexadecimal DATA WORD
 *   --------------------CC-   is a two digit hexadecimal byte based CHECKSUM
 *   ----------------------;   the HEX record end marker
 *
 *
 *   HEX record data layout - details:
 *   =================================
 *
 *   The HEX record structure uses ASCII values 0x30 through 0x46 only. Any
 *   other ASCII character will be ignored.
 *
 *   BB - is a two digit hexadecimal NUMBER representing the number of data
 *   bytes as used in HHHH. Because HHHH is always a WORD size, the BB number
 *   is always an even number.
 *
 *   AAAA - is a four digit hexadecimal ADDRESS representing the starting
 *   address of the data record. Format is high byte first followed by low byte.
 *   The address is doubled because this format only supports 8-bits (to find
 *   the real PICmicro address, simply divide the value AAAA by 2). For those
 *   record types which do not use an address, the address will be ignored.
 *
 *   HHHH - is a four digit hexadecimal DATA WORD. Format is low byte followed
 *   by high byte. There will be BB/2 data words following TT.
 *
 *   CC - is a two digit hexadecimal CHECKSUM that is the two's complement of
 *   the sum of all the preceding bytes in the line record.
 *
 *   TT - is a two digit record TYPE, which are defined as:
 *
 *   '00' - data record to WRITE BB/2 words starting at address AAAA
 *   '10' - data record to FORMAT  32 words starting at address AAAA
 *   '20' - data recard to READ    32 words starting at address AAAA
 *
 *   'F0' - get Boot Loader Status (show FLAG)
 *   'F1' - get User Program Status (check ID and CRC)
 *   'F2' - reboot MCU
 *   'FF' - clear the status flag
 *
 *   Once the complete HEX record was recieved (end marker detected), parsed
 *   and its checksum was found to be correct, it will be executed. A status
 *   reply will be send back to the master.
 *
 *   Only a sequential HEX RECORD START (:) and HEX RECORD STOP (;) in a string
 *   of characters will trigger a status reply. The status reply will be within
 *   milliseconds.
 *
 *
 *   HEX record sizes:
 *   =================
 *
 *   The theoretical maximum data containing one HEX record is:
 *   2 + 4 + 2 + 2 * 254 + 2 = 518 chars
 *   1 + 2 + 1 +     254 + 1 = 259 bytes
 *                   254     = 127 words
 *
 *   The maximum as defined for this boot loader (using 8-bit indices):
 *   2 + 4 + 2 + 2 * 122 + 2 = 254 chars
 *   1 + 2 + 1 +     122 + 1 = 127 bytes
 *                   122     = 61  words
 *
 *   The most economic design for one HEX record is as follows.
 *   Based on a flash block size of 32 words for the PIC16F1789.
 *   2 + 4 + 2 + 2 *  64 + 2 = 138 chars
 *   1 + 2 + 1 +      64 + 1 = 69  bytes
 *                    64     = 32  words
 *
 *   The Microchip XC8 compiler HEX record is:
 *   2 + 4 + 2 + 2 *  16 + 2 = 42  chars
 *   1 + 2 + 1 +      16 + 1 = 21  bytes
 *                    16     = 8   words
 *
 *   The minimum data containing one HEX record is:
 *   2 + 4 + 2 + 2 *   0 + 2 = 10  chars
 *   1 + 2 + 1 +       0 + 1 = 5   bytes
 *                     0     = 0   words
 *
 *
 *   Example HEX records:
 *   ====================
 *   :00100010E0;            Erase 32 words at address 0x0800 (0x1000/2)
 *   :00200010D0;            Erase 32 words at address 0x1000 (0x2000/2)
 *   :0009C02017;            Get Boot Loader Record (32 words size)
 *   :007FC020A1;            Get User Program Record (32 words size)
 *   :000000F010;            Get Boot Loader Status (FLAG status)
 *   :000000F10F;            Get User Program Status (check ID and CRC)
 *   :000000F20E;            Reboot MCU
 *   :000000FF01;            Clear the Status Flag
 *
 *
 *   ANSWER record layout:
 *   =====================
 *
 *   The BOOT LOADER will reply each recieved HEX record with an answer. This
 *   answer could be a series of data words (depending the record type), but
 *   always a single status byte.
 *
 *   :HHHH...HHHH;
 *   -------------
 *   :------------             the ANSWER record start marker
 *   -HHHH...HHHH-             is a four digit hexadecimal DATA WORD
 *   ------------;             the ANSWER record end marker
 *
 *   :XX;
 *   ----
 *   :---                      the ANSWER record start marker
 *   -XX-                      the STATUS byte, OKAY == 0; ERROR > 0;
 *   ---;                      the ANSWER record end marker
 *
 *
 ******************************************************************************/

/*******************************************************************************
 ** Start Header Definition
 ******************************************************************************/

#define HEX_REC_INIT         0x3A                // ':' - Start of HEX record
#define HEX_REC_DAT0         0x30                // '0' - HEX record data - 0
#define HEX_REC_DAT1         0x31                // '1' - HEX record data - 1
#define HEX_REC_DAT2         0x32                // '2' - HEX record data - 2
#define HEX_REC_DAT3         0x33                // '3' - HEX record data - 3
#define HEX_REC_DAT4         0x34                // '4' - HEX record data - 4
#define HEX_REC_DAT5         0x35                // '5' - HEX record data - 5
#define HEX_REC_DAT6         0x36                // '6' - HEX record data - 6
#define HEX_REC_DAT7         0x37                // '7' - HEX record data - 7
#define HEX_REC_DAT8         0x38                // '8' - HEX record data - 8
#define HEX_REC_DAT9         0x39                // '9' - HEX record data - 9
#define HEX_REC_DATA         0x41                // 'A' - HEX record data - A
#define HEX_REC_DATB         0x42                // 'B' - HEX record data - B
#define HEX_REC_DATC         0x43                // 'C' - HEX record data - C
#define HEX_REC_DATD         0x44                // 'D' - HEX record data - D
#define HEX_REC_DATE         0x45                // 'E' - HEX record data - E
#define HEX_REC_DATF         0x46                // 'F' - HEX record data - F
#define HEX_REC_STOP         0x3B                // ';' - End of HEX record

/* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */

#define HEX_REC_MIN          10                  // HEX MIN record size in chars
#define HEX_BYT_MIN          5                   // HEX MIN data size in bytes
#define HEX_WRD_MIN          0                   // HEX MIN value size in words

#define HEX_REC_MAX          254                 // HEX MAX record size in chars
#define HEX_BYT_MAX          127                 // HEX MAX data size in bytes
#define HEX_WRD_MAX          61                  // HEX MAX value size in words

/* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */

#define HEX_TYPE_WRITE       0x00                // HEX record write to flash
#define HEX_TYPE_ERASE       0x10                // HEX record format flash (32 words block)
#define HEX_TYPE_READ        0x20                // HEX record read from flash (32 words block)

#define HEX_TYPE_CHK_BLS     0xF0                // Check Boot Loader Request STATUS
#define HEX_TYPE_CHK_UPS     0xF1                // Check User Program ID and CRC STATUS
#define HEX_TYPE_REBOOT      0xF2                // Reboot the MCU
#define HEX_TYPE_STS_CLR     0xFF                // Clear the STATUS Flag

/*******************************************************************************
 ** Typedefs
 ******************************************************************************/

typedef struct TypeHexRecord
{
  uint8_t Chars[HEX_REC_MAX];
  uint8_t CharIdx;
  uint8_t Bytes[HEX_BYT_MAX];
  uint8_t ByteIdx;
  bool Parsed;
};

/******************************************************************************/

typedef struct TypeFlashRecord
{
  uint8_t Count;
  uint16_t Address;
  uint8_t Type;
  uint16_t Words[HEX_WRD_MAX];
};

/*******************************************************************************
 ** User Function Prototypes
 ******************************************************************************/

enum RETURN BootLoaderRequested (void);
enum RETURN ConfirmUserProgram (void);
void ExecuteBootLoader (void);

/*******************************************************************************
 ** END-O-HEADER
 ******************************************************************************/
