/*******************************************************************************
 *                                      *                                      *
 *    HOLDUP BV                         *    STOGGER BV                        *
 *    Nachtegaalstraat 8                *    Haagveld 1A                       *
 *    5988 NG  Helden                   *    5981 PK  Panningen                *
 *    The Netherlands                   *    The Netherlands                   *
 *                                      *                                      *
 *******************************************************************************
 *                                      *                                      *
 *    Aubert Dupont                     *                                      *
 *    aubert@holdup.nl                  *                                      *
 *                                      *                                      *
 *******************************************************************************
 *                                                                             *
 *    Project    -   STOGGER - E-41-2000 - Ventilator PAL                      *
 *    Author     -   Aubert Dupont                                             *
 *    Build Tool -   MPLAB X IDE v4.15 & MPLAB XC8 v1.45                       *
 *    Revision   -   1.01                                                      *
 *    Date       -   2020-04-01                                                *
 *    Function   -   User definition file                                      *
 *    File       -   user.h                                                    *
 *                                                                             *
 *******************************************************************************
 *                                                                             *
 *    (C)  2020  HOLDUP BV         ALL RIGHTS RESERVED                         *
 *    (C)  2020  STOGGER BV        ALL RIGHTS RESERVED                         *
 *                                                                             *
 ******************************************************************************/

/*******************************************************************************
 *
 *   Documentation / Functional description
 *
 *******************************************************************************
 *
 *   For Communication from MASTER to VENTILATOR, a modified version of the
 *   HEX record data format is used.
 *
 *   The communication specifications:
 *
 *   Clock:   Asynchronous
 *   Mode:    HALF-duplex
 *   Speed:   9600 baud
 *   Size:    8-bits transmission
 *   Parity:  None
 *   Stop:    1-bit
 *
 *
 *   HEX record data layout:
 *   =======================
 *
 *   The HEX record structure used, is a modified version of the Microchip
 *   or Intel standard. The modification consists of different record begin
 *   and end markers; and additional type records; and no address.
 *
 *   <BBTTHH...HHCC>   one single HEX record
 *   ---------------
 *   <--------------   the HEX record start marker
 *   -BB------------   the NUMBER of data bytes in this record
 *   ---TT----------   is a two digit record TYPE
 *   -----HH...HH---   is a two digit hexadecimal DATA BYTE
 *   ------------CC-   is a two digit hexadecimal byte based CHECKSUM
 *   -------------->   the HEX record end marker
 *
 *
 *   HEX record data layout - details:
 *   =================================
 *
 *   The HEX record structure uses ASCII values 0x30 through 0x46 only. Any
 *   other ASCII character will be ignored.
 *
 *   BB - is a two digit hexadecimal NUMBER representing the number of data
 *   bytes as used in HH.
 *
 *   HH - is a two digit hexadecimal DATA BYTE. Format is low byte followed
 *   by high byte (little endian).
 *
 *   CC - is a two digit hexadecimal CHECKSUM that is the two's complement of
 *   the sum of all the preceding bytes in the line record.
 *
 *   TT - is a two digit record TYPE, which are defined as:
 *
 *   '00' - '3F' - Reserved
 *
 *   '40' - '6F' - GLOBAL settings
 *   '70' - '7F  - MOTION general SET
 *   '80' - '8F' - MOTION profiles SET
 *   '90' - '9F' - MOTION profile path SET
 *   'A0' - 'AF' - MOTION general GET
 *   'B0' - 'BF' - MOTION profiles GET
 *   'C0' - 'CF' - MOTION profile path GET
 *
 *   Once the complete HEX record was recieved (end marker detected), parsed
 *   and its checksum was found to be correct, it will be executed. A status
 *   reply will be send back to the master.
 *
 *   Only a sequential HEX RECORD START (<) and HEX RECORD STOP (>) in a string
 *   of characters will trigger a status reply. The status reply will be within
 *   milliseconds.
 *
 *
 *   HEX record sizes:
 *   =================
 *
 *   The theoretical maximum data containing one HEX record is:
 *   2 + 2 + 2 * 255 + 2 = 516 chars
 *   1 + 1 +     255 + 1 = 258 bytes
 *
 *   The maximum as defined for this module (using 8-bit indices & available RAM):
 *   2 + 2 + 2 * 64 + 2 = 134 chars
 *   1 + 1 +     64 + 1 = 67 bytes
 *
 *   The minimum data containing one HEX record is:
 *   2 + 2 + 2 *   0 + 2 = 6   chars
 *   1 + 1 +       0 + 1 = 3   bytes
 *
 *
 *   Example HEX records:
 *   ====================
 *
 *   <0040C0>  --------------------------  Get User Program Information: Watchdog Reboot and ID type
 *   <0041BF>  --------------------------  Get User Program Version: Major and Minor
 *
 *   <004EB2>  --------------------------  Reboot into User Program (restart)
 *   <004FB1>  --------------------------  Reboot into Boot Loader
 *
 *   <0050B0>  --------------------------  Get HOME pin status
 *   <0051AF>  --------------------------  Get FAULT pin status
 *   <0052AE>  --------------------------  Get ERROR status byte (and clear it)
 *   <0053AD>  --------------------------  Get STATE machine byte
 * 
 *   <0058A8>  --------------------------  Get number of profiles & paths
 *   <0059A7>  --------------------------  Get current position
 *   <005AA6>  --------------------------  Get current time count (1/1000s)
 *
 *   <0060A0>  --------------------------  Motion: Calculate Profiles
 *   <00619F>  --------------------------  Motion: Calculate Profiles and Save Settings
 *   <00629E>  --------------------------  Motion: Resume motion
 *   <00639D>  --------------------------  Motion: Enter Pause state
 * 
 *   <006898>  --------------------------  Dumps 256 bytes of EEPROM onto UART
 *   <006997>  --------------------------  Load Motion Profiles from EEPROM
 *   <006A96>  --------------------------  Store Motion Profiles in EEPROM
 *   <016BAAEA>  -----------------------   Wipe 256 bytes of EEPROM with [0xAA]
 *   <016BFF95>  -----------------------   Wipe 256 bytes of EEPROM with [0xFF]
 *   <026CFE553F>  ---------------------   Store one byte in EEPROM [add:0xFE, val:0x55]
 *   <026CFE0193>  ---------------------   Store one byte in EEPROM [add:0xFE, val:0x01]
 *
 *   <0170018E>  ------------------------  Set Motion General RampsNum to 1
 *   <0171800E>  ------------------------  Set Motion General RampsDen to 128
 *
 *   <028000017D>  ----------------------  Set Motion Profile [0] MoveMode to 01
 *   <028001017C>  ----------------------  Set Motion Profile [1] MoveMode to 01
 *   <028200FF7D>  ----------------------  Set Motion Profile [0] PowerReg to 0xFF
 *   <038300E0FF9B>  --------------------  Set Motion Profile [0] SpeedReg to 0xFFE0
 * 
 *   <04900020ABCDD4>  ------------------  Set Motion Profile [0] Ramp Table [32] Poistion to 0xCDAB
 * 
 *   <079F000000000000005A>  ------------  Set Motion Profile [0] Ramp Table [0] to zero
 *
 *   <00A45C>  --------------------------  Get Motion General RampVMax setting
 * 
 *   <00AF51>  --------------------------  Get Motion General settings
 *
 *   <01B0004F>  ------------------------  Get Motion Profile [0] MoveMode
 *   <01B0014E>  ------------------------  Get Motion Profile [1] MoveMode
 *   <01B3004C>  ------------------------  Get Motion Profile [0] SpeedReg
 *
 *   <01BF0040>  ------------------------  Get Motion Profile [0] complete profile settings
 *   <01BF013F>  ------------------------  Get Motion Profile [1] complete profile settings
 * 
 *   <02CF00002F>  ----------------------  Get Motion Profile [0] Ramp Table [0] settings
 *   <02CF00200F>  ----------------------  Get Motion Profile [0] Ramp Table [32] settings
 *   <02CF0040EF>  ----------------------  Get Motion Profile [0] Ramp Table [64] settings
 *
 *
 *   MOTION:
 *   =======
 *
 *   Maximum travel is defined by a 16 bit unsigned int.
 *   Hence a 65.536 (2^16) full travel step-distance is available.
 *
 *   Movements can be performed in two MODES:
 *   - FIXED velocity
 *   - TIMED velocity
 *
 *   Movements can be executed in two WAYS:
 *   - CONSTANT velocity
 *   - RAMPING profile
 *
 *   Movements in CONSTANT velocity are executed immediately using the fixed
 *   clock frequency, whereas RAMPING calculated a specific 3rd order (constant
 *   jerk) profile, which results in smoother run.
 *
 *   Please refer to the seperate calculations sheet "Ramping calculations.ods"
 *
 *
 *   EEPROM memory layout:
 *   =====================
 *
 *   There are 256 bytes of EEPROM in the PIC16F1788
 *
 *   ||=====================================||
 *   || OFFSET | Description                ||
 *   ||=====================================||
 *
 *   ||=====================================||
 *   ||   0x00 | EEPROM content checksum    ||
 *   ||-------------------------------------||
 *   ||   0x01 | Motion Profiles            ||
 *   ||-------------------------------------||
 *   ||   0xFE | Major Firmware Version     ||
 *   ||   0xFF | Minor Firmware Version     ||
 *   ||=====================================||
 *
 *   The Firmware version is used to check the EEPROM validity and/or
 *   data compatibility; additionally the checksum, over the 0x01..0xFF EEPROM
 *   address range, is used to confirm the data integrity.
 *
 *
 ******************************************************************************/

/*******************************************************************************
 ** Communication Definitions
 ******************************************************************************/

#define HEX_REC_INIT         0x3C                // '<' - Start of HEX record
#define HEX_REC_DAT0         0x30                // '0' - HEX record data - 0
#define HEX_REC_DAT1         0x31                // '1' - HEX record data - 1
#define HEX_REC_DAT2         0x32                // '2' - HEX record data - 2
#define HEX_REC_DAT3         0x33                // '3' - HEX record data - 3
#define HEX_REC_DAT4         0x34                // '4' - HEX record data - 4
#define HEX_REC_DAT5         0x35                // '5' - HEX record data - 5
#define HEX_REC_DAT6         0x36                // '6' - HEX record data - 6
#define HEX_REC_DAT7         0x37                // '7' - HEX record data - 7
#define HEX_REC_DAT8         0x38                // '8' - HEX record data - 8
#define HEX_REC_DAT9         0x39                // '9' - HEX record data - 9
#define HEX_REC_DATA         0x41                // 'A' - HEX record data - A
#define HEX_REC_DATB         0x42                // 'B' - HEX record data - B
#define HEX_REC_DATC         0x43                // 'C' - HEX record data - C
#define HEX_REC_DATD         0x44                // 'D' - HEX record data - D
#define HEX_REC_DATE         0x45                // 'E' - HEX record data - E
#define HEX_REC_DATF         0x46                // 'F' - HEX record data - F
#define HEX_REC_STOP         0x3E                // '>' - End of HEX record

/* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */

#define HEX_REC_MIN          6                   // HEX MIN record size in chars
#define HEX_BYT_MIN          3                   // HEX MIN data size in bytes

#define HEX_REC_MAX          134                 // HEX MAX record size in chars
#define HEX_BYT_MAX          67                  // HEX MAX data size in bytes

/*******************************************************************************
 ** Commando Types (TT HEX record byte)
 ******************************************************************************/

#define CMD_GET_INFO         0x40                // Get User Program Information
#define CMD_GET_VERSION      0x41                // Get User Program Version

/* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */

#define CMD_PIC_REBOOT       0x4E                // Reboot into User Program (restart)
#define CMD_BL_REQUEST       0x4F                // Reboot into Boot Loader

/******************************************************************************/

#define CMD_GET_HOME         0x50                // Get HOME pin status
#define CMD_GET_FAULT        0x51                // Get FAULT pin status
#define CMD_GET_ERROR        0x52                // Get ERROR status (and clear)
#define CMD_GET_STATE        0x53                // Get STATE machine status

/* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */

#define CMD_GET_SIZE         0x58                // Get number of profiles & paths
#define CMD_GET_CURPOS       0x59                // Get current position
#define CMD_GET_CURCNT       0x5A                // Get current time count (1/1000s)

/******************************************************************************/

#define CMD_MOTION_CALC      0x60                // Motion: Calculate Profiles
#define CMD_MOTION_SAVE      0x61                // Motion: Calculate Profiles and Save Settings
#define CMD_MOTION_EXEC      0x62                // Motion: Resume motion
#define CMD_MOTION_WAIT      0x63                // Motion: Enter Pause state

/* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */

#define CMD_EEPROM_DUMP      0x68                // Dumps 256 bytes of EEPROM onto UART
#define CMD_EEPROM_LOAD      0x69                // Load Motion Profiles from EEPROM
#define CMD_EEPROM_SAVE      0x6A                // Store Motion Profiles in EEPROM
#define CMD_EEPROM_WIPE      0x6B                // Wipe 256 bytes of EEPROM with [value]
#define CMD_EEPROM_BYTE      0x6C                // Store one byte on address in EEPROM [address, value]

/******************************************************************************/

#define CMD_SET_JERKDVDR     0x70                // Set JerkDvdr (uint16_t)
#define CMD_SET_ACCLDVDR     0x71                // Set AcclDvdr (uint16_t)
#define CMD_SET_VELODVDR     0x72                // Set VeloDvdr (uint16_t)
#define CMD_SET_RAMPVMAX     0x73                // Set RampVMax (uint16_t)
#define CMD_SET_CONSVMAX     0x74                // Set ConsVMax (uint16_t)
#define CMD_SET_PLACEMIN     0x75                // Set PlaceMin (uint16_t)
#define CMD_SET_PLACEMAX     0x76                // Set PlaceMax (uint16_t)
#define CMD_SET_HOMEOFFS     0x77                // Set HomeOffs (uint16_t)
#define CMD_SET_HOMECHKV     0x78                // Set HomeChkV (uint16_t)

/* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */

#define CMD_SET_GENERAL      0x7F                // Set General Settings (typedef)

/******************************************************************************/

#define CMD_SET_MOVEMODE     0x80                // Set P[i].MoveMode (uint8_t)
#define CMD_SET_MOVETYPE     0x81                // Set P[i].MoveType (uint8_t)
#define CMD_SET_POWERREG     0x82                // Set P[i].PowerReg (uint8_t)
#define CMD_SET_SPEEDREG     0x83                // Set P[i].SpeedReg (uint16_t)
#define CMD_SET_TIMEDREG     0x84                // Set P[i].TimedReg (uint16_t)
#define CMD_SET_PLACECUR     0x85                // Set P[i].PlaceCur (uint16_t)
#define CMD_SET_PLACEDES     0x86                // Set P[i].PlaceDes (uint16_t)
#define CMD_SET_TIMEWAIT     0x87                // Set P[i].TimeWait (uint16_t)

/* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */

#define CMD_SET_PROFILE      0x8F                // Set Profile[i] Settings (typedef)

/******************************************************************************/

#define CMD_SET_POSITION     0x90                // Set P[i].R[j].Position (uint16_t)
#define CMD_SET_TIMERVAL     0x91                // Set P[i].R[j].TimerVal (uint16_t)
#define CMD_SET_FLAGDATA     0x92                // Set P[i].R[j].FlagData (uint8_t:1)
#define CMD_SET_FLAGLIST     0x93                // Set P[i].R[j].FlagList (uint8_t:1)

/* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */

#define CMD_SET_RAMPTBL      0x9F                // Set Profile[i].Ramp[j] Settings (typedef)

/******************************************************************************/

#define CMD_GET_JERKDVDR     0xA0                // Get JerkDvdr (uint16_t)
#define CMD_GET_ACCLDVDR     0xA1                // Get AcclDvdr (uint16_t)
#define CMD_GET_VELODVDR     0xA2                // Get VeloDvdr (uint16_t)
#define CMD_GET_RAMPVMAX     0xA3                // Get RampVMax (uint16_t)
#define CMD_GET_CONSVMAX     0xA4                // Get ConsVMax (uint16_t)
#define CMD_GET_PLACEMIN     0xA5                // Get PlaceMin (uint16_t)
#define CMD_GET_PLACEMAX     0xA6                // Get PlaceMax (uint16_t)
#define CMD_GET_HOMEOFFS     0xA7                // Get HomeOffs (uint16_t)
#define CMD_GET_HOMECHKV     0xA8                // Get HomeChkV (uint16_t)

/* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */

#define CMD_GET_GENERAL      0xAF                // Get General Settings (typedef)

/******************************************************************************/

#define CMD_GET_MOVEMODE     0xB0                // Get P[i].MoveMode (uint8_t)
#define CMD_GET_MOVETYPE     0xB1                // Get P[i].MoveType (uint8_t)
#define CMD_GET_POWERREG     0xB2                // Get P[i].PowerReg (uint8_t)
#define CMD_GET_SPEEDREG     0xB3                // Get P[i].SpeedReg (uint16_t)
#define CMD_GET_TIMEDREG     0xB4                // Get P[i].TimedReg (uint16_t)
#define CMD_GET_PLACECUR     0xB5                // Get P[i].PlaceCur (uint16_t)
#define CMD_GET_PLACEDES     0xB6                // Get P[i].PlaceCur (uint16_t)
#define CMD_GET_TIMEWAIT     0xB7                // Get P[i].TimeWait (uint16_t)

/* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */

#define CMD_GET_PROFILE      0xBF                // Get Profile[i] Settings (typedef)

/******************************************************************************/

#define CMD_GET_POSITION     0xC0                // Get P[i].R[j].Position (uint16_t)
#define CMD_GET_TIMERVAL     0xC1                // Get P[i].R[j].TimerVal (uint16_t)
#define CMD_GET_FLAGDATA     0xC2                // Get P[i].R[j].FlagData (uint8_t:1)
#define CMD_GET_FLAGLIST     0xC3                // Get P[i].R[j].FlagList (uint8_t:1)

/* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */

#define CMD_GET_RAMPTBL      0xCF                // Get Profile[i].Ramp[j] Settings (typedef)

/*******************************************************************************
 ** Motion Definitions
 ******************************************************************************/

#define FREQ_CLK             (FREQ_OSC/4)        // The TIMER 1 CLK frequency
#define FREQ_MIN             (FREQ_CLK/UINT16_MAX)
#define FREQ_DIV             1000                // The SPEED divider (1/1000)

/* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */

#define TMR1_START           0xFF00U             // Default init value timer 1

/******************************************************************************/

#define RMP_SIZE             128                 // Ramp profile table SIZE
#define RMP_FULL             64                  // Ramp profile table FULL
#define RMP_HALF             32                  // Ramp profile table HALF
#define RMP_ROOT             1024                // Ramp profile table HALF

/* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */

#define RMP_SEG0             0                   // Ramp profile jerk SEGMENT 0
#define RMP_SEG1             (RMP_HALF-1)        // Ramp profile jerk SEGMENT 1
#define RMP_SEG2             RMP_HALF            // Ramp profile jerk SEGMENT 2

/******************************************************************************/

#define DEFAULT_JERKDVDR     RMP_ROOT            // Default Jerk divider
#define DEFAULT_ACCLDVDR     RMP_ROOT            // Default Accelaration divider
#define DEFAULT_VELODVDR     1024                // Default Velocity divider
#define DEFAULT_RAMPVMAX     0xFFA0              // Default Max Ramping Vel (minimum timer) == 84kHz
#define DEFAULT_CONSVMAX     0xFFD0              // Default Max Constant Vel (minimum timer) == 170kHz
#define DEFAULT_PLACEMIN     0x0000              // Default place minimum (range)
#define DEFAULT_PLACEMAX     0x9768              // Default place maximum (range)
#define DEFAULT_HOMEOFFS     0x0200              // Default Homeing offset position
#define DEFAULT_HOMECHKV     0x0400              // Default Homeing check value position

/******************************************************************************/

#define PROFILE_MAX          2                   // Max number of Profiles
#define PROFILE_MASK         0x01                // Max number of Profiles

/* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  */

#define PROFILE_HOME_UP      0                   // Index of the HOME_UP profile
#define PROFILE_HOME_DW      1                   // Index of the HOME_UP profile
#define PROFILE_INH_MOVE     0                   // Index of the INHALE MOVE profile
#define PROFILE_EXH_MOVE     1                   // Index of the EXHALE MOVE profile

/*******************************************************************************
 ** EEPROM Definitions
 ******************************************************************************/

#define EEP_ADD_CHECKSUM     0x00                // EEPROM 0x01..0xFF content Checksum
#define EEP_ADD_PROFILES     0x01                // EEPROM Motion Profiles Address
#define EEP_ADD_FW_MAJOR     0xFE                // EEPROM Firmware Major Address
#define EEP_ADD_FW_MINOR     0xFF                // EEPROM Firmware Minor Address

/*******************************************************************************
 ** Typedefs
 ******************************************************************************/

typedef enum ERRORS                              // Errors are ORed
{
  ERROR_NONE = 0,                                // No errors
  ERROR_HOME = 1,                                // Homing error
  ERROR_MOVE = 2,                                // Movement error
  ERROR_UART = 4,                                // UART error
  ERROR_DRV  = 8,                                // Driver error (temperature, overcurrent)
} ;

/******************************************************************************/

typedef enum STATES
{
  STATE_BOOT          = 0,
  STATE_HOME_UP_INIT  = 1,
  STATE_HOME_UP_EXEC  = 2,
  STATE_HOME_DW_INIT  = 3,
  STATE_HOME_DW_EXEC  = 4,
  STATE_HOMING_ERROR  = 5,
  STATE_PROFILES_LOAD = 7,
  STATE_PROFILES_CALC = 8,
  STATE_PAUSE_INIT    = 10,
  STATE_PAUSE_EXEC    = 11,
  STATE_INH_MOVE_INIT = 15,
  STATE_INH_MOVE_EXEC = 16,
  STATE_INH_WAIT_INIT = 20,
  STATE_INH_WAIT_EXEC = 21,
  STATE_EXH_MOVE_INIT = 25,
  STATE_EXH_MOVE_EXEC = 26,
  STATE_EXH_WAIT_INIT = 30,
  STATE_EXH_WAIT_EXEC = 31,
} ;

/******************************************************************************/

typedef enum TypeMoveMode
{
  MOTION_VELO = 0,                               // Velocity motion mode.
  MOTION_TIME = 1,                               // Timed motion mode.
} ;

/******************************************************************************/

typedef enum TypeMoveType
{
  MOTION_CONS = 0,                               // Constant velocity motion type.
  MOTION_RAMP = 1,                               // Ramping profile motion type.
} ;

/******************************************************************************/

typedef enum TypeMoveCalc
{
  MOTION_DONE = 0,                               // Motion request finished
  MOTION_CALC = 1,                               // Motion calculation request
  MOTION_SAVE = 2,                               // Motion calc and save request
  MOTION_WAIT = 3,                               // Motion pause request
} ;

/******************************************************************************/

typedef struct TypeMotionGeneral
{
  uint16_t                   JerkDvdr;           // Ramping calculations Jerk divider
  uint16_t                   AcclDvdr;           // Ramping calculations Acceleration divider
  uint16_t                   VeloDvdr;           // Ramping calculations Velocity divider
  uint16_t                   RampVMax;           // Maximum Timer value for Ramping
  uint16_t                   ConsVMax;           // Maximum Timer value for Constant
  uint16_t                   PlaceMin;           // Workarea minimum
  uint16_t                   PlaceMax;           // Workarea maximum
  uint16_t                   HomeOffs;           // Homeing Offset
  uint16_t                   HomeChkV;           // Homeing Check Value
} ;

/******************************************************************************/

typedef struct TypeMotionProfile
{
  enum     TypeMoveMode      MoveMode;           // Velocity or Timed mode
  enum     TypeMoveType      MoveType;           // Constant or Ramping mode
  uint8_t                    PowerReg;           // Motor Power register
  uint16_t                   SpeedReg;           // Speed timer register
  uint16_t                   TimedReg;           // Timed velocity register
  uint16_t                   PlaceCur;           // Current Position register
  uint16_t                   PlaceDes;           // Destination Position register
  uint16_t                   TimeWait;           // Time to Wait in 1/1000 seconds
} ;

/******************************************************************************/

typedef struct TypeRampTable
{
  uint16_t                   Position;           // Position for the new Timer
  uint16_t                   TimerVal;           // New Timer value
  uint8_t                    FlagData : 1;       // This record contains valid data
  uint8_t                    FlagList : 1;       // This is the last record flag
} ;

/******************************************************************************/

typedef struct TypeMotionPath
{
  enum     TypeMoveType      MoveType;           // Constant or Ramping mode
  struct   TypeRampTable     Ramp[RMP_SIZE + 1]; // The ramping profile table
  uint8_t                    PowerReg;           // Motor Power register
  uint16_t                   TimerVal;           // The actual TIMER register value
  uint16_t                   PlaceCur;           // Current Position register
  uint16_t                   PlaceDes;           // Destination Position register
  uint16_t                   TimeWait;           // Time to Wait in 1/1000 seconds
  uint8_t                    Direction : 1;      // The actual DIR port value
  uint8_t                    Valid : 1;          // Is this path valid
} ;

/******************************************************************************/

typedef struct TypeDataRecord
{
  uint8_t                    Chars[HEX_REC_MAX]; // Buffer for the UART data
  uint8_t                    CharIdx;            // UART data buffer pointer
  uint8_t                    Bytes[HEX_BYT_MAX]; // The converted data bytes
  uint8_t                    ByteIdx;            // Data byte index
  uint8_t                    Count;              // Number of data bytes
  uint8_t                    Type;               // The actual command
  bool                       Parsed;             // If the UART data was parsed
} ;

/*******************************************************************************
 ** User Function Prototypes
 ******************************************************************************/

void InitiateApplication (void);
void ExecuteApplication (void);

/*******************************************************************************
 ** END-O-HEADER
 ******************************************************************************/
